//###################################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//###################################################################################
#ifndef __EZ_ILDA_SEGMENT_DEFINITIONS__
#define __EZ_ILDA_SEGMENT_DEFINITIONS__

//###################################################################################
#include "ez_bmp.hpp"
#include "ez_real_segment.hpp"

//###################################################################################
class ez_ilda_frame_set; //predeclaration

//###################################################################################
class ez_ilda_segment : public vector<ez_ilda_point>
{
public:
    ez_ilda_segment()
                     : palette_index      (EZ_ILDA_DEFAULT),
                       error              (EZ_ILDA_OK     )      {}
    //-------------------------------------------------------------------------------
    ez_ilda_segment(const int& palette_index) // empty segment
                     : palette_index      (palette_index),
                       error              (EZ_ILDA_OK   )        {}
    //-------------------------------------------------------------------------------
    ez_ilda_segment(const int& palette_index, int not_used) // not empty segment
                     : palette_index      (palette_index),
                       error              (EZ_ILDA_OK   )
                    {
                        push_back(ez_ilda_point());
                        push_back(ez_ilda_point());
                    }
    //-------------------------------------------------------------------------------
    ez_ilda_segment(const ez_ilda_segment& segment)
                     : palette_index      (segment.palette_index),
                       error              (EZ_ILDA_OK           )
                    {
                        clear();
                        insert(end(), segment.begin(), segment.end());
                    }
    //-------------------------------------------------------------------------------
    ez_ilda_segment(ez_ilda_point from, ez_ilda_point to)
                     : palette_index      (EZ_ILDA_DEFAULT),
                       error              (EZ_ILDA_OK     )
                    {   // 3D line constructor
                        int steps = distance_points(to, from);
                        if(steps)
                        {
                            ez_3D_float  _to        (to),
                                         _from      (from),
                                         difference (_to - _from),
                                         delta      (difference / steps);
                            //-------------------------------------------------------
                            for(int i = 1; i <= steps; i++)
                                push_back(ez_ilda_point( ez_3D_short(_from + (delta * i)),
                                                         (ez_color)to,
                                                         to.k,
                                                         to.c
                                                       )
                                         );
                            //-------------------------------------------------------
                        }
                    }
    //-------------------------------------------------------------------------------
    ez_ilda_segment(ez_ilda_point p1,
                    ez_ilda_point p2,
                    float         rotation,
                    string        font,
                    string        text
                   );   // 3D text constructor
    //-------------------------------------------------------------------------------
    ez_ilda_segment(ez_ilda_point center,
                    ez_ilda_point arc_start,
                    float         arc_angle // OK degrees, whatever!
                   );   // 3D circular arc constructor
    //-------------------------------------------------------------------------------
    ez_ilda_segment(ez_ilda_point center,
                    ez_ilda_point arc_start,
                    float         arc_angle,
                    float         radii_ratio
                   );   // 3D elliptical arc constructor
    //-------------------------------------------------------------------------------
    ez_ilda_segment(ez_ilda_point center,
                    ez_ilda_point first_vertex,
                    int           number_of_sides
                   );   // 3D polygon constructor
    //-------------------------------------------------------------------------------
   ~ez_ilda_segment()    {    clear();    };
    //-------------------------------------------------------------------------------
    inline ez_ilda_segment&  operator += (const ez_ilda_segment& segment)
                                {
                                    if(palette_index != segment.palette_index)
                                        promote_to_true_color();
                                    insert(end(), segment.begin(), segment.end());
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline ez_ilda_segment&  operator += (const ez_ilda_point& point)
                                {
                                    push_back(point);
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline ez_ilda_segment&  operator =  (const ez_ilda_segment& segment)
                                {
                                    clear();
                                    //palette_index = segment.palette_index;
                                    insert(begin(), segment.begin(), segment.end());
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline bool is_2D() const
                    {
                        if(number_of_points() > 1)
                        {
                            for(int i = 0; i < number_of_points(); i++)
                                if(at(i).z != 0)
                                    return false;
                        }
                        return true;
                    }
    //-------------------------------------------------------------------------------
    inline ez_ilda_point  first_lit_point() const
                {
                    for(int i = 1; i < number_of_points(); i++)
                        if(at(i).is_lit())
                            return at(i);
                    return ez_ilda_point(0);
                }
    //-------------------------------------------------------------------------------
    inline int number_of_points() const 
                {
                    return (int)size();
                }
    //-------------------------------------------------------------------------------
    inline int first_lit_point_index() const
                {
                    for(int i = 1; i < number_of_points(); i++)
                        if(at(i).is_lit())
                            return i;
                    return -1;
                }
    //-------------------------------------------------------------------------------
    inline ez_ilda_point first_lit_anchor() const
                            {
                                for(int i = 1; i < number_of_points(); i++)
                                    if(at(i).is_lit())
                                        return at(i - 1);
                                return ez_ilda_point(0);
                            }
    //-------------------------------------------------------------------------------
    inline int first_lit_anchor_index() const
                {
                    for(int i = 1; i < number_of_points(); i++)
                        if(at(i).is_lit())
                            return (i - 1);
                    return -1;
                }
    //-------------------------------------------------------------------------------
    inline ez_ilda_point last_lit_anchor() const
                            {
                                for(int i = number_of_points() - 1; i > 0; i--)
                                    if(at(i).is_lit())
                                        return at(i - 1);
                                return ez_ilda_point(0);
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_point last_lit_point() const
                            {
                                for(int i = number_of_points() - 1; i > 0; i--)
                                    if(at(i).is_lit())
                                        return at(i);
                                return ez_ilda_point(0);
                            }
    //-------------------------------------------------------------------------------
    inline int last_lit_point_index() const
                {
                    for(int i = number_of_points() - 1; i > 0; i--)
                        if(at(i).is_lit())
                            return i;
                    return -1;
                }
    //-------------------------------------------------------------------------------
    inline int number_of_lit_points() const
                {
                    int count = 0;
                    for(int i = 1; i < number_of_points(); i++)
                        if(at(i).is_lit())
                            count++;
                    return count;
                }
    //-------------------------------------------------------------------------------
    inline int number_of_blank_points() const
                {
                    int count = 1; //first point is always an anchor
                    for(int i = 1; i < number_of_points(); i++)
                        if(at(i).is_blank())
                            count++;
                    return count;
                }
    //-------------------------------------------------------------------------------
    inline int number_of_unique_colors() const
                {
                    if(number_of_points() > 1)
                    {
                        int           i, j;
                        ez_palette  palette;
                        palette.push_back((ez_color)first_lit_point());
                        for(i = first_lit_point_index() + 1; i < number_of_points(); i++)
                        {
                            for(j = 0; j < palette.number_of_colors(); j++)
                            {
                                if(palette[j] == (ez_color)at(i))
                                    break;
                            }
                            if(j == palette.number_of_colors())
                                palette.push_back(at(i));
                        }
                        return palette.number_of_colors();
                    }
                }
    //-------------------------------------------------------------------------------
    inline ez_palette as_ez_palette() const
                        {
                            ez_palette palette(number_of_points());
                            for(int i = 0; i < number_of_points(); i++)
                                palette[i] = (ez_color)(at(i));
                            return palette;
                        }
    //-------------------------------------------------------------------------------
    inline ez_ilda_segment& remove_point(int point_index)
                            {
                                if(    point_index >= 0
                                    && point_index <  (int)size()
                                    && size()
                                  )
                                {
                                    if(size() == 1)
                                        clear();
                                    int              i;
                                    ez_ilda_segment  segment;
                                    for(i = 0; i < point_index; i++)
                                        segment += at(i);
                                    for(i = point_index + 1; i < (int)size(); i++)
                                        segment += at(i);
                                    *this = segment;
                                }
                                return *this;
                            }
    //-------------------------------------------------------------------------------
    ez_3D_short        segment_front                 (                               ) const ;
    ez_3D_short        segment_back                  (                               ) const ;
    ez_3D_short        segment_top                   (                               ) const ;
    ez_3D_short        segment_bottom                (                               ) const ;
    ez_3D_short        segment_right                 (                               ) const ;
    ez_3D_short        segment_left                  (                               ) const ;
    short              segment_height                (                               ) const ;
    short              segment_width                 (                               ) const ;
    short              segment_depth                 (                               ) const ;
    short              segment_size                  (                               ) const ;
    bool               is_closed_polygon             (                               ) const ;
    ez_ilda_segment&   reverse                       (                               )       ;
    ez_ilda_segment    shortest_path_of_segments     (                               )       ;
    void               unblank_all                   (                               )       ;
    //void               remove_added_points           (                               )       ;
    void               minimize_blank_points         (                               )       ;
    void               remove_dwell_points           (                               )       ;
    void               minimize_vectors              (                               )       ;
    void               minimize_points               (                               )       ;
    ez_ilda_segment&   reorder_segments              (                               )       ;
    ez_ilda_segment&   fracture_segments             (                               )       ;
    ez_ilda_segment&   bond_segments                 (                               )       ;
    ez_ilda_segment&   add_new_point                 (int  point_index               )       ;
    ez_ilda_segment&   break_segment                 (int& point_index               )       ;
    ez_ilda_segment&   connect_the_dots              (int p1, int p2                 )       ;
    double             next_point_angle              (int point                      ) const ;
    double             total_angle                   (                               ) const ;
    double             max_angle                     (                               ) const ;
    double             next_point_distance           (int point                      ) const ;
    double             total_distance                (                               ) const ;
    double             max_distance                  (                               ) const ;
    void               add_dwell_points              (                               )       ;
    void               add_span_points               (                               )       ;
    void               flip                          (int plane                      )       ;
    void               quarter_turn                  (int plane, int turns           )       ;
    void               z_order_points                (unsigned short span            )       ;
    void               flatten_z                     (                               )       ;
    int                rotate                        (ez_3D_float a                  )       ;
    int                rotate_around_origin          (ez_3D_float a                  )       ;
    int                rotate_around_point           (ez_3D_float p, ez_3D_float a   )       ;
    int                rotate_around_point_x         (ez_3D_float p, float a         )       ;
    int                rotate_around_point_y         (ez_3D_float p, float a         )       ;
    int                rotate_around_point_z         (ez_3D_float p, float a         )       ;
    int                rotate_around_origin          (                               )       ;
    void               ripple                        (int   direction,
                                                      float amplitude,
                                                      float freq     ,
                                                      float phase                    )       ;
    void               accept_palette                (int index                      )       ;
    void               accept_target_palette         (                               )       ;
    void               best_match_palette            (int index                      )       ;
    void               best_match_target_palette     (                               )       ;
    void               rainbow_recolor               (int effect, float factor       )       ;
    void               rotate_colors                 (                               )       ;
    bool               find_rgb_in_palette           (const ez_palette& palette      )       ;
    void               set_rgb_from_palette          (                               )       ;
    void               set_palette_to_332            (                               )       ;
    void               sync_rgb_and_palette          (                               )       ;
    void               bit_reduce_to_palette         (                               )       ;
    void               promote_to_true_color         (                               )       ;
    void               color_over_ezfb_bitmap        (struct ezfb_bitmap* bmp        )       ;
    int                move                          (ez_3D_float d                  )       ;
    int                move                          (                               )       ;
    int                scale                         (ez_3D_float s                  )       ;
    int                scale_around_point            (ez_3D_float p, ez_3D_float s   )       ;
    int                scale_around_origin           (ez_3D_float s                  )       ;
    int                scale_around_origin           (                               )       ;
    ez_3D_float        find_center                   (                               ) const ;
    int                number_of_segments            (                               ) const ;
    bool               find_segment                  (int index, int& start, int& end) const ;
    bool               find_segment                  ( int point_index,
                                                       int& start,
                                                       int& end,
                                                       int& segment_index
                                                     )                                 const ;
    ez_ilda_segment    find_segment                  (int index                      ) const ;
    ez_ilda_frame_set  find_segments                 (                               ) const ;
    ez_3D_float        find_segment_center           (int index                      ) const ;
    int                segment_index_of_point        (int index                      ) const ;
    int                move_segment                  (int index, ez_3D_float d       )       ;
    int                rotate_segment                (int index, ez_3D_float a       )       ;
    int                rotate_segment_around_origin  (int index, ez_3D_float a       )       ;
    int                scale_segment                 (int index, ez_3D_float m       )       ;
    int                scale_segment_around_origin   (int index, ez_3D_float m       )       ;
    void               to_ofstream_wave              (ofstream&  out,
                                                      const char& end_of_frame = 0x00,
                                                      const char& unique_frame = 0x00
                                                     )                                 const ;
    //-------------------------------------------------------------------------------
    int                palette_index      ,
                       error              ;
    //-------------------------------------------------------------------------------
};

//###################################################################################
ez_ilda_segment blank_segment();
ez_ilda_segment NULL_segment ();

//###################################################################################
#endif

//###################################################################################
/////////////////////////////////////////////////////////////////////////////////////
//###################################################################################



