//############################################################################
// ~ International Laser Display Association ~
// ~     ILDA file format driver in C++      ~
// ~             built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#include "ez_ilda.hpp"
    
//############################################################################
ez_palette ez_palette_set::palette_picker(int index) const
{
    if(index == EZ_TRUE_COLOR)
        return at(ez_ilda_pointer->target_palette_index);
    if(index < 0)
        index = number_of_palettes() - 1;
    else if(index >= number_of_palettes())
        index = 0;
    return at(index);
};

//############################################################################
void ez_palette_set::render_palette_in_ezfb_bitmap() const
{
    int   i,
          j,
          y_line = 8 * ez_ilda_pointer->font_size_factor,
          y_off =   EZ_PALETTE_MAX * ez_ilda_pointer->font_size_factor
                  + EZ_STATS_DISPLAY_OFFSET * ez_ilda_pointer->font_size_factor,

          x_off = BMP.yres + EZ_STATS_DISPLAY_OFFSET * ez_ilda_pointer->font_size_factor;
    //------------------------------------------------------------------------
    if(palette_index != EZ_TRUE_COLOR) // 8 bit frame palette
    {
        for(i = 0; i < palette_picker(palette_index).number_of_colors(); i++)
            for(j = 0; j < ez_ilda_pointer->font_size_factor; j++)
                bmp_put_line_fast( &BMP,
                                   x_off,
                                   i * ez_ilda_pointer->font_size_factor + j,
                                   x_off + 50,
                                   i * ez_ilda_pointer->font_size_factor + j,
                                   BMP.make_rgb(palette_picker(palette_index)[i].r,
                                                palette_picker(palette_index)[i].g,
                                                palette_picker(palette_index)[i].b
                                               )
                                 );
    }
    //------------------------------------------------------------------------
    for(i = 0; i < palette_picker(target_palette_index).number_of_colors(); i++) // target palette
        for(j = 0; j < ez_ilda_pointer->font_size_factor; j++)
            bmp_put_line_fast( &BMP,
                               x_off + 70,
                               i * ez_ilda_pointer->font_size_factor + j,
                               x_off + 120,
                               i * ez_ilda_pointer->font_size_factor + j,
                               BMP.make_rgb(palette_picker(target_palette_index)[i].r,
                                            palette_picker(target_palette_index)[i].g,
                                            palette_picker(target_palette_index)[i].b
                                           )
                             );
    //------------------------------------------------------------------------
    if(ez_ilda_pointer->show_cursors) // selected_palette_color cursor
    {
        bmp_put_line_fast( &BMP,
                           x_off + 60,
                           0,
                           x_off + 60,
                           EZ_PALETTE_MAX * ez_ilda_pointer->font_size_factor,
                           BMP.make_rgb(palette_picker(palette_index)[selected_palette_color].r,
                                        palette_picker(palette_index)[selected_palette_color].g,
                                        palette_picker(palette_index)[selected_palette_color].b
                                       )
                         );
    //------------------------------------------------------------------------
        for(j = 0; j < ez_ilda_pointer->font_size_factor; j++) // selected color cursor
            bmp_put_line_fast( &BMP,
                               x_off + 40,
                               selected_palette_color * ez_ilda_pointer->font_size_factor + j,
                               x_off + 80,
                               selected_palette_color * ez_ilda_pointer->font_size_factor + j,
                               0x00ffffff
                             );
    }
    //------------------------------------------------------------------------
    if(palette_index == EZ_TRUE_COLOR)
        bmp_printf(   &BMP // current frame palette name
                    , x_off
                    , y_off
                    , 0x00000000
                    , 0x00ffffff
                    , 0
                    , ez_ilda_pointer->font_size_factor
                    , "-24-bit-"
                  );
    else
        bmp_printf(   &BMP
                    , x_off
                    , y_off
                    , 0x00ffffff
                    , 0x00000000
                    , 0
                    , ez_ilda_pointer->font_size_factor
                    , "%s"
                    , palette_picker(palette_index).name.c_str()
                  );

    bmp_printf(   &BMP // target palette name
                , x_off + 130
                , 0
                , 0x00ffffff
                , 0x00000000
                , 1
                , ez_ilda_pointer->font_size_factor
                , "%s"
                , palette_picker(target_palette_index).name.c_str()
              );

    if(ez_ilda_pointer->show_cursors)
    {        
        y_off += y_line;
        bmp_printf(   &BMP // selected color blue value
                    , x_off
                    , y_off
                    , BMP.make_rgb(0,0,255)
                    , 0x00000000
                    , 0
                    , ez_ilda_pointer->font_size_factor
                    , "            %3d"
                    , palette_picker(palette_index)[selected_palette_color].b
                  );
        bmp_printf(   &BMP // selected color green value
                    , x_off
                    , y_off
                    , BMP.make_rgb(0,255,0)
                    , 0x00000000
                    , 0
                    , ez_ilda_pointer->font_size_factor
                    , "        %3d"
                    , palette_picker(palette_index)[selected_palette_color].g
                  );
        bmp_printf(   &BMP // selected color red value
                    , x_off
                    , y_off
                    , BMP.make_rgb(255,0,0)
                    , 0x00000000
                    , 0
                    , ez_ilda_pointer->font_size_factor
                    , "    %3d"
                    , palette_picker(palette_index)[selected_palette_color].r
                  );
        bmp_printf(   &BMP // selected color index
                    , x_off
                    , y_off
                    , 0x00ffffff
                    , 0x00000000
                    , 0
                    , ez_ilda_pointer->font_size_factor
                    , "%3d"
                    , selected_palette_color
                  );
    }
}

//############################################################################
void ez_palette_set::populate_palette_set()
{
    ez_palette  palette;
    u_char        r = 0,
                  g = 0,
                  b = 0,
                  shift;
    int           i,
                  hue;
    //------------------------------------------------------------------------
    palette.name = "Default_";
    palette.first = 0;
    palette.push_back(ez_color(0xff, 0x00, 0x00));
    palette.push_back(ez_color(0xff, 0x10, 0x00));
    palette.push_back(ez_color(0xff, 0x20, 0x00));
    palette.push_back(ez_color(0xff, 0x30, 0x00));
    palette.push_back(ez_color(0xff, 0x40, 0x00));
    palette.push_back(ez_color(0xff, 0x50, 0x00));
    palette.push_back(ez_color(0xff, 0x60, 0x00));
    palette.push_back(ez_color(0xff, 0x70, 0x00));
    palette.push_back(ez_color(0xff, 0x80, 0x00));
    palette.push_back(ez_color(0xff, 0x90, 0x00));
    palette.push_back(ez_color(0xff, 0xa0, 0x00));
    palette.push_back(ez_color(0xff, 0xb0, 0x00));
    palette.push_back(ez_color(0xff, 0xc0, 0x00));
    palette.push_back(ez_color(0xff, 0xd0, 0x00));
    palette.push_back(ez_color(0xff, 0xe0, 0x00));
    palette.push_back(ez_color(0xff, 0xf0, 0x00));
    palette.push_back(ez_color(0xff, 0xff, 0x00));
    palette.push_back(ez_color(0xe0, 0xff, 0x00));
    palette.push_back(ez_color(0xc0, 0xff, 0x00));
    palette.push_back(ez_color(0xa0, 0xff, 0x00));
    palette.push_back(ez_color(0x80, 0xff, 0x00));
    palette.push_back(ez_color(0x60, 0xff, 0x00));
    palette.push_back(ez_color(0x40, 0xff, 0x00));
    palette.push_back(ez_color(0x20, 0xff, 0x00));
    palette.push_back(ez_color(0x00, 0xff, 0x00));
    palette.push_back(ez_color(0x00, 0xff, 0x20));
    palette.push_back(ez_color(0x00, 0xff, 0x40));
    palette.push_back(ez_color(0x00, 0xff, 0x60));
    palette.push_back(ez_color(0x00, 0xff, 0x80));
    palette.push_back(ez_color(0x00, 0xff, 0xa0));
    palette.push_back(ez_color(0x00, 0xff, 0xc0));
    palette.push_back(ez_color(0x00, 0xff, 0xe0));
    palette.push_back(ez_color(0x00, 0x82, 0xff));
    palette.push_back(ez_color(0x00, 0x72, 0xff));
    palette.push_back(ez_color(0x00, 0x68, 0xff));
    palette.push_back(ez_color(0x0a, 0x60, 0xff));
    palette.push_back(ez_color(0x00, 0x52, 0xff));
    palette.push_back(ez_color(0x00, 0x4a, 0xff));
    palette.push_back(ez_color(0x00, 0x40, 0xff));
    palette.push_back(ez_color(0x00, 0x20, 0xff));
    palette.push_back(ez_color(0x00, 0x00, 0xff));
    palette.push_back(ez_color(0x20, 0x00, 0xff));
    palette.push_back(ez_color(0x40, 0x00, 0xff));
    palette.push_back(ez_color(0x60, 0x00, 0xff));
    palette.push_back(ez_color(0x80, 0x00, 0xff));
    palette.push_back(ez_color(0xa0, 0x00, 0xff));
    palette.push_back(ez_color(0xe0, 0x00, 0xff));
    palette.push_back(ez_color(0xff, 0x00, 0xff));
    palette.push_back(ez_color(0xff, 0x20, 0xff));
    palette.push_back(ez_color(0xff, 0x40, 0xff));
    palette.push_back(ez_color(0xff, 0x60, 0xff));
    palette.push_back(ez_color(0xff, 0x80, 0xff));
    palette.push_back(ez_color(0xff, 0xa0, 0xff));
    palette.push_back(ez_color(0xff, 0xc0, 0xff));
    palette.push_back(ez_color(0xff, 0xe0, 0xff));
    palette.push_back(ez_color(0xff, 0xff, 0xff)); // white
    palette.white = palette.number_of_colors() - 1;
    palette.push_back(ez_color(0xff, 0xe0, 0xe0));
    palette.push_back(ez_color(0xff, 0xff, 0xff));
    palette.push_back(ez_color(0xff, 0xa0, 0xa0));
    palette.push_back(ez_color(0xff, 0x80, 0x80));
    palette.push_back(ez_color(0xff, 0x60, 0x60));
    palette.push_back(ez_color(0xff, 0x40, 0x40));
    palette.push_back(ez_color(0xff, 0x20, 0x20)); // last unique
    palette.last  = palette.number_of_colors() - 1;
    palette.push_back(ez_color(0x80, 0x80, 0x80)); // gray
    palette.gray  = palette.number_of_colors() - 1;
    palette.push_back(ez_color(0x00, 0x00, 0x00)); // black
    palette.black = palette.number_of_colors() - 1;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "X29_____";
    palette.push_back(ez_color(0x00, 0x00, 0x00)); // black
    palette.black = palette.number_of_colors() - 1;
    palette.push_back(ez_color(0x00, 0x00, 0x00));
    palette.push_back(ez_color(0xff, 0x00, 0x00));
    palette.first = palette.number_of_colors() - 1;
    palette.push_back(ez_color(0xff, 0x00, 0x00));
    palette.push_back(ez_color(0xff, 0xff, 0x00));
    palette.push_back(ez_color(0xff, 0xff, 0x00));
    palette.push_back(ez_color(0x00, 0xff, 0x00));
    palette.push_back(ez_color(0x00, 0xff, 0x00));
    palette.push_back(ez_color(0x80, 0x80, 0x00));
    palette.push_back(ez_color(0x80, 0x80, 0x00));
    palette.push_back(ez_color(0x00, 0x00, 0xff));
    palette.push_back(ez_color(0x00, 0x00, 0xff));
    palette.push_back(ez_color(0xff, 0x00, 0xff));
    palette.push_back(ez_color(0xff, 0x00, 0xff));
    palette.push_back(ez_color(0xff, 0xff, 0xff)); // white
    palette.white = palette.number_of_colors() - 1;
    palette.push_back(ez_color(0xff, 0xff, 0xff));
    palette.push_back(ez_color(0x00, 0xff, 0xff));
    palette.push_back(ez_color(0x00, 0xff, 0xff));
    palette.push_back(ez_color(0x00, 0xff, 0xff));
    palette.push_back(ez_color(0x00, 0xff, 0xff)); //last unique
    palette.last  = palette.number_of_colors() - 1;
    palette.push_back(ez_color(0x80, 0x80, 0x80)); // gray
    palette.gray  = palette.number_of_colors() - 1;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "ILDA_std";
    palette.push_back(ez_color(0x00, 0x00, 0x00)); // black
    palette.black = palette.number_of_colors() - 1;
    palette.push_back(ez_color(0xff, 0xff, 0xff)); // white
    palette.white = palette.number_of_colors() - 1;
    palette.first = palette.number_of_colors() - 1;  // first
    palette.push_back(ez_color(0xff, 0x00, 0x00));
    palette.first = palette.number_of_colors() - 1;
    palette.push_back(ez_color(0xff, 0xff, 0x00));
    palette.push_back(ez_color(0x00, 0xff, 0x00));
    palette.push_back(ez_color(0x00, 0xff, 0xff));
    palette.push_back(ez_color(0x00, 0x00, 0xff));
    palette.push_back(ez_color(0xff, 0x00, 0xff));
    palette.push_back(ez_color(0xff, 0x80, 0x80));
    palette.push_back(ez_color(0xff, 0x8c, 0x80));
    palette.push_back(ez_color(0xff, 0x97, 0x80));
    palette.push_back(ez_color(0xff, 0xa3, 0x80));
    palette.push_back(ez_color(0xff, 0xae, 0x80));
    palette.push_back(ez_color(0xff, 0xba, 0x80));
    palette.push_back(ez_color(0xff, 0xc5, 0x80));
    palette.push_back(ez_color(0xff, 0xd1, 0x80));
    palette.push_back(ez_color(0xff, 0xdc, 0x80));
    palette.push_back(ez_color(0xff, 0xe8, 0x80));
    palette.push_back(ez_color(0xff, 0xf3, 0x80));
    palette.push_back(ez_color(0xff, 0xff, 0x80));
    palette.push_back(ez_color(0xf3, 0xff, 0x80));
    palette.push_back(ez_color(0xe8, 0xff, 0x80));
    palette.push_back(ez_color(0xdc, 0xff, 0x80));
    palette.push_back(ez_color(0xd1, 0xff, 0x80));
    palette.push_back(ez_color(0xc5, 0xff, 0x80));
    palette.push_back(ez_color(0xba, 0xff, 0x80));
    palette.push_back(ez_color(0xae, 0xff, 0x80));
    palette.push_back(ez_color(0xa3, 0xff, 0x80));
    palette.push_back(ez_color(0x97, 0xff, 0x80));
    palette.push_back(ez_color(0x8c, 0xff, 0x80));
    palette.push_back(ez_color(0x80, 0xff, 0x80));
    palette.push_back(ez_color(0x80, 0xff, 0x8c));
    palette.push_back(ez_color(0x80, 0xff, 0x97));
    palette.push_back(ez_color(0x80, 0xff, 0xa3));
    palette.push_back(ez_color(0x80, 0xff, 0xae));
    palette.push_back(ez_color(0x80, 0xff, 0xba));
    palette.push_back(ez_color(0x80, 0xff, 0xc5));
    palette.push_back(ez_color(0x80, 0xff, 0xd1));
    palette.push_back(ez_color(0x80, 0xff, 0xdc));
    palette.push_back(ez_color(0x80, 0xff, 0xe8));
    palette.push_back(ez_color(0x80, 0xff, 0xf3));
    palette.push_back(ez_color(0x80, 0xff, 0xff));
    palette.push_back(ez_color(0x80, 0xf3, 0xff));
    palette.push_back(ez_color(0x80, 0xe8, 0xff));
    palette.push_back(ez_color(0x80, 0xdc, 0xff));
    palette.push_back(ez_color(0x80, 0xd1, 0xff));
    palette.push_back(ez_color(0x80, 0xc5, 0xff));
    palette.push_back(ez_color(0x80, 0xba, 0xff));
    palette.push_back(ez_color(0x80, 0xae, 0xff));
    palette.push_back(ez_color(0x80, 0xa3, 0xff));
    palette.push_back(ez_color(0x80, 0x97, 0xff));
    palette.push_back(ez_color(0x80, 0x8c, 0xff));
    palette.push_back(ez_color(0x80, 0x80, 0xff));
    palette.push_back(ez_color(0x8c, 0x80, 0xff));
    palette.push_back(ez_color(0x97, 0x80, 0xff));
    palette.push_back(ez_color(0xa3, 0x80, 0xff));
    palette.push_back(ez_color(0xae, 0x80, 0xff));
    palette.push_back(ez_color(0xba, 0x80, 0xff));
    palette.push_back(ez_color(0xc5, 0x80, 0xff));
    palette.push_back(ez_color(0xd1, 0x80, 0xff));
    palette.push_back(ez_color(0xdc, 0x80, 0xff));
    palette.push_back(ez_color(0xe8, 0x80, 0xff));
    palette.push_back(ez_color(0xf3, 0x80, 0xff));
    palette.push_back(ez_color(0xff, 0x80, 0xff));
    palette.push_back(ez_color(0xff, 0x80, 0xf3));
    palette.push_back(ez_color(0xff, 0x80, 0xe8));
    palette.push_back(ez_color(0xff, 0x80, 0xdc));
    palette.push_back(ez_color(0xff, 0x80, 0xd1));
    palette.push_back(ez_color(0xff, 0x80, 0xc5));
    palette.push_back(ez_color(0xff, 0x80, 0xba));
    palette.push_back(ez_color(0xff, 0x80, 0xae));
    palette.push_back(ez_color(0xff, 0x80, 0xa3));
    palette.push_back(ez_color(0xff, 0x80, 0x97));
    palette.push_back(ez_color(0xff, 0x80, 0x8c));
    palette.push_back(ez_color(0xff, 0x00, 0x00));
    palette.push_back(ez_color(0xff, 0x17, 0x00));
    palette.push_back(ez_color(0xff, 0x2e, 0x00));
    palette.push_back(ez_color(0xff, 0x46, 0x00));
    palette.push_back(ez_color(0xff, 0x5d, 0x00));
    palette.push_back(ez_color(0xff, 0x74, 0x00));
    palette.push_back(ez_color(0xff, 0x8b, 0x00));
    palette.push_back(ez_color(0xff, 0xa2, 0x00));
    palette.push_back(ez_color(0xff, 0xb9, 0x00));
    palette.push_back(ez_color(0xff, 0xd1, 0x00));
    palette.push_back(ez_color(0xff, 0xe8, 0x00));
    palette.push_back(ez_color(0xff, 0xff, 0x00));
    palette.push_back(ez_color(0xe8, 0xff, 0x00));
    palette.push_back(ez_color(0xd1, 0xff, 0x00));
    palette.push_back(ez_color(0xb9, 0xff, 0x00));
    palette.push_back(ez_color(0xa2, 0xff, 0x00));
    palette.push_back(ez_color(0x8b, 0xff, 0x00));
    palette.push_back(ez_color(0x74, 0xff, 0x00));
    palette.push_back(ez_color(0x5d, 0xff, 0x00));
    palette.push_back(ez_color(0x46, 0xff, 0x00));
    palette.push_back(ez_color(0x2e, 0xff, 0x00));
    palette.push_back(ez_color(0x17, 0xff, 0x00));
    palette.push_back(ez_color(0x00, 0xff, 0x00));
    palette.push_back(ez_color(0x00, 0xff, 0x17));
    palette.push_back(ez_color(0x00, 0xff, 0x2e));
    palette.push_back(ez_color(0x00, 0xff, 0x46));
    palette.push_back(ez_color(0x00, 0xff, 0x5d));
    palette.push_back(ez_color(0x00, 0xff, 0x74));
    palette.push_back(ez_color(0x00, 0xff, 0x8b));
    palette.push_back(ez_color(0x00, 0xff, 0xa2));
    palette.push_back(ez_color(0x00, 0xff, 0xb9));
    palette.push_back(ez_color(0x00, 0xff, 0xd1));
    palette.push_back(ez_color(0x00, 0xff, 0xe8));
    palette.push_back(ez_color(0x00, 0xff, 0xff));
    palette.push_back(ez_color(0x00, 0xe8, 0xff));
    palette.push_back(ez_color(0x00, 0xd1, 0xff));
    palette.push_back(ez_color(0x00, 0xb9, 0xff));
    palette.push_back(ez_color(0x00, 0xa2, 0xff));
    palette.push_back(ez_color(0x00, 0x8b, 0xff));
    palette.push_back(ez_color(0x00, 0x74, 0xff));
    palette.push_back(ez_color(0x00, 0x5d, 0xff));
    palette.push_back(ez_color(0x00, 0x46, 0xff));
    palette.push_back(ez_color(0x00, 0x2e, 0xff));
    palette.push_back(ez_color(0x00, 0x17, 0xff));
    palette.push_back(ez_color(0x00, 0x00, 0xff));
    palette.push_back(ez_color(0x17, 0x00, 0xff));
    palette.push_back(ez_color(0x2e, 0x00, 0xff));
    palette.push_back(ez_color(0x46, 0x00, 0xff));
    palette.push_back(ez_color(0x5d, 0x00, 0xff));
    palette.push_back(ez_color(0x74, 0x00, 0xff));
    palette.push_back(ez_color(0x8b, 0x00, 0xff));
    palette.push_back(ez_color(0xa2, 0x00, 0xff));
    palette.push_back(ez_color(0xb9, 0x00, 0xff));
    palette.push_back(ez_color(0xd1, 0x00, 0xff));
    palette.push_back(ez_color(0xe8, 0x00, 0xff));
    palette.push_back(ez_color(0xff, 0x00, 0xff));
    palette.push_back(ez_color(0xff, 0x00, 0xe8));
    palette.push_back(ez_color(0xff, 0x00, 0xd1));
    palette.push_back(ez_color(0xff, 0x00, 0xb9));
    palette.push_back(ez_color(0xff, 0x00, 0xa2));
    palette.push_back(ez_color(0xff, 0x00, 0x8b));
    palette.push_back(ez_color(0xff, 0x00, 0x74));
    palette.push_back(ez_color(0xff, 0x00, 0x5d));
    palette.push_back(ez_color(0xff, 0x00, 0x46));
    palette.push_back(ez_color(0xff, 0x00, 0x2e));
    palette.push_back(ez_color(0xff, 0x00, 0x17));
    palette.push_back(ez_color(0x80, 0x00, 0x00));
    palette.push_back(ez_color(0x80, 0x0c, 0x00));
    palette.push_back(ez_color(0x80, 0x17, 0x00));
    palette.push_back(ez_color(0x80, 0x23, 0x00));
    palette.push_back(ez_color(0x80, 0x2f, 0x00));
    palette.push_back(ez_color(0x80, 0x3a, 0x00));
    palette.push_back(ez_color(0x80, 0x46, 0x00));
    palette.push_back(ez_color(0x80, 0x51, 0x00));
    palette.push_back(ez_color(0x80, 0x5d, 0x00));
    palette.push_back(ez_color(0x80, 0x69, 0x00));
    palette.push_back(ez_color(0x80, 0x74, 0x00));
    palette.push_back(ez_color(0x80, 0x80, 0x00));
    palette.push_back(ez_color(0x74, 0x80, 0x00));
    palette.push_back(ez_color(0x69, 0x80, 0x00));
    palette.push_back(ez_color(0x5d, 0x80, 0x00));
    palette.push_back(ez_color(0x51, 0x80, 0x00));
    palette.push_back(ez_color(0x46, 0x80, 0x00));
    palette.push_back(ez_color(0x3a, 0x80, 0x00));
    palette.push_back(ez_color(0x2f, 0x80, 0x00));
    palette.push_back(ez_color(0x23, 0x80, 0x00));
    palette.push_back(ez_color(0x17, 0x80, 0x00));
    palette.push_back(ez_color(0x0c, 0x80, 0x00));
    palette.push_back(ez_color(0x00, 0x80, 0x00));
    palette.push_back(ez_color(0x00, 0x80, 0x0c));
    palette.push_back(ez_color(0x00, 0x80, 0x17));
    palette.push_back(ez_color(0x00, 0x80, 0x23));
    palette.push_back(ez_color(0x00, 0x80, 0x2f));
    palette.push_back(ez_color(0x00, 0x80, 0x3a));
    palette.push_back(ez_color(0x00, 0x80, 0x46));
    palette.push_back(ez_color(0x00, 0x80, 0x51));
    palette.push_back(ez_color(0x00, 0x80, 0x5d));
    palette.push_back(ez_color(0x00, 0x80, 0x69));
    palette.push_back(ez_color(0x00, 0x80, 0x74));
    palette.push_back(ez_color(0x00, 0x80, 0x80));
    palette.push_back(ez_color(0x00, 0x74, 0x80));
    palette.push_back(ez_color(0x00, 0x69, 0x80));
    palette.push_back(ez_color(0x00, 0x5d, 0x80));
    palette.push_back(ez_color(0x00, 0x51, 0x80));
    palette.push_back(ez_color(0x00, 0x46, 0x80));
    palette.push_back(ez_color(0x00, 0x3a, 0x80));
    palette.push_back(ez_color(0x00, 0x2f, 0x80));
    palette.push_back(ez_color(0x00, 0x23, 0x80));
    palette.push_back(ez_color(0x00, 0x17, 0x80));
    palette.push_back(ez_color(0x00, 0x0c, 0x80));
    palette.push_back(ez_color(0x00, 0x00, 0x80));
    palette.push_back(ez_color(0x0c, 0x00, 0x80));
    palette.push_back(ez_color(0x17, 0x00, 0x80));
    palette.push_back(ez_color(0x23, 0x00, 0x80));
    palette.push_back(ez_color(0x2f, 0x00, 0x80));
    palette.push_back(ez_color(0x3a, 0x00, 0x80));
    palette.push_back(ez_color(0x46, 0x00, 0x80));
    palette.push_back(ez_color(0x51, 0x00, 0x80));
    palette.push_back(ez_color(0x5d, 0x00, 0x80));
    palette.push_back(ez_color(0x69, 0x00, 0x80));
    palette.push_back(ez_color(0x74, 0x00, 0x80));
    palette.push_back(ez_color(0x80, 0x00, 0x80));
    palette.push_back(ez_color(0x80, 0x00, 0x74));
    palette.push_back(ez_color(0x80, 0x00, 0x69));
    palette.push_back(ez_color(0x80, 0x00, 0x5d));
    palette.push_back(ez_color(0x80, 0x00, 0x51));
    palette.push_back(ez_color(0x80, 0x00, 0x46));
    palette.push_back(ez_color(0x80, 0x00, 0x3a));
    palette.push_back(ez_color(0x80, 0x00, 0x2f));
    palette.push_back(ez_color(0x80, 0x00, 0x23));
    palette.push_back(ez_color(0x80, 0x00, 0x17));
    palette.push_back(ez_color(0x80, 0x00, 0x0c));
    palette.push_back(ez_color(0xff, 0xc0, 0xc0));
    palette.push_back(ez_color(0xff, 0x40, 0x40));
    palette.push_back(ez_color(0xc0, 0x00, 0x00));
    palette.push_back(ez_color(0x40, 0x00, 0x00));
    palette.push_back(ez_color(0xff, 0xff, 0xc0));
    palette.push_back(ez_color(0xff, 0xff, 0x40));
    palette.push_back(ez_color(0xc0, 0xc0, 0x00));
    palette.push_back(ez_color(0x40, 0x40, 0x00));
    palette.push_back(ez_color(0xc0, 0xff, 0xc0));
    palette.push_back(ez_color(0x40, 0xff, 0x40));
    palette.push_back(ez_color(0x00, 0xc0, 0x00));
    palette.push_back(ez_color(0x00, 0x40, 0x00));
    palette.push_back(ez_color(0xc0, 0xff, 0xff));
    palette.push_back(ez_color(0x40, 0xff, 0xff));
    palette.push_back(ez_color(0x00, 0xc0, 0xc0));
    palette.push_back(ez_color(0x00, 0x40, 0x40));
    palette.push_back(ez_color(0xc0, 0xc0, 0xff));
    palette.push_back(ez_color(0x40, 0x40, 0xff));
    palette.push_back(ez_color(0x00, 0x00, 0xc0));
    palette.push_back(ez_color(0x00, 0x00, 0x40));
    palette.push_back(ez_color(0xff, 0xc0, 0xff));
    palette.push_back(ez_color(0xff, 0x40, 0xff));
    palette.push_back(ez_color(0xc0, 0x00, 0xc0));
    palette.push_back(ez_color(0x40, 0x00, 0x40));
    palette.push_back(ez_color(0xff, 0x60, 0x60));
    palette.push_back(ez_color(0xff, 0xff, 0xff)); // white
    palette.push_back(ez_color(0xf5, 0xf5, 0xf5));
    palette.push_back(ez_color(0xeb, 0xeb, 0xeb));
    palette.push_back(ez_color(0xe0, 0xe0, 0xe0));
    palette.push_back(ez_color(0xd5, 0xd5, 0xd5));
    palette.push_back(ez_color(0xcb, 0xcb, 0xcb));
    palette.push_back(ez_color(0xc0, 0xc0, 0xc0));
    palette.push_back(ez_color(0xb5, 0xb5, 0xb5));
    palette.push_back(ez_color(0xab, 0xab, 0xab));
    palette.push_back(ez_color(0xa0, 0xa0, 0xa0));
    palette.push_back(ez_color(0x95, 0x95, 0x95));
    palette.push_back(ez_color(0x8b, 0x8b, 0x8b));
    palette.push_back(ez_color(0x80, 0x80, 0x80)); // gray
    palette.gray  = palette.number_of_colors() - 1;
    palette.push_back(ez_color(0x75, 0x75, 0x75));
    palette.push_back(ez_color(0x6b, 0x6b, 0x6b));
    palette.push_back(ez_color(0x60, 0x60, 0x60));
    palette.push_back(ez_color(0x55, 0x55, 0x55));
    palette.push_back(ez_color(0x4b, 0x4b, 0x4b));
    palette.push_back(ez_color(0x40, 0x40, 0x40));
    palette.push_back(ez_color(0x35, 0x35, 0x35));
    palette.push_back(ez_color(0x2b, 0x2b, 0x2b));
    palette.push_back(ez_color(0x20, 0x20, 0x20));
    palette.push_back(ez_color(0x15, 0x15, 0x15));
    palette.push_back(ez_color(0x0b, 0x0b, 0x0b));
    palette.last  = palette.number_of_colors() - 1;  // last
    palette.push_back(ez_color(0x00, 0x00, 0x00)); // black
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "DXF_____";
    palette.push_back(ez_color(0x00, 0x00, 0x00));
    palette.black = palette.number_of_colors() - 1;  // black
    palette.push_back(ez_color(0xff, 0x00, 0x00));
    palette.first = palette.number_of_colors() - 1;  // first
    palette.push_back(ez_color(0xff, 0xff, 0x00));
    palette.push_back(ez_color(0x00, 0xff, 0x00));
    palette.push_back(ez_color(0x00, 0xff, 0xff));
    palette.push_back(ez_color(0x00, 0x00, 0xff));
    palette.push_back(ez_color(0xff, 0x00, 0xff));
    palette.push_back(ez_color(0xff, 0xff, 0xff));
    palette.push_back(ez_color(0x7f, 0x7f, 0x7f));
    palette.gray  = palette.number_of_colors() - 1;  // gray
    palette.push_back(ez_color(0xbf, 0xbf, 0xbf));
    palette.push_back(ez_color(0xff, 0x00, 0x00));
    palette.push_back(ez_color(0xff, 0x7f, 0x7f));
    palette.push_back(ez_color(0xa5, 0x00, 0x00));
    palette.push_back(ez_color(0xa5, 0x52, 0x52));
    palette.push_back(ez_color(0x7f, 0x00, 0x00));
    palette.push_back(ez_color(0x7f, 0x3f, 0x3f));
    palette.push_back(ez_color(0x4c, 0x00, 0x00));
    palette.push_back(ez_color(0x4c, 0x26, 0x26));
    palette.push_back(ez_color(0x26, 0x00, 0x00));
    palette.push_back(ez_color(0x26, 0x13, 0x13));
    palette.push_back(ez_color(0xff, 0x3f, 0x00));
    palette.push_back(ez_color(0xff, 0x9f, 0x7f));
    palette.push_back(ez_color(0xa5, 0x29, 0x00));
    palette.push_back(ez_color(0xa5, 0x67, 0x52));
    palette.push_back(ez_color(0x7f, 0x1f, 0x00));
    palette.push_back(ez_color(0x7f, 0x4f, 0x3f));
    palette.push_back(ez_color(0x4c, 0x13, 0x00));
    palette.push_back(ez_color(0x4c, 0x2f, 0x26));
    palette.push_back(ez_color(0x26, 0x09, 0x00));
    palette.push_back(ez_color(0x26, 0x17, 0x13));
    palette.push_back(ez_color(0xff, 0x7f, 0x00));
    palette.push_back(ez_color(0xff, 0xbf, 0x7f));
    palette.push_back(ez_color(0xa5, 0x52, 0x00));
    palette.push_back(ez_color(0xa5, 0x7c, 0x52));
    palette.push_back(ez_color(0x7f, 0x3f, 0x00));
    palette.push_back(ez_color(0x7f, 0x5f, 0x3f));
    palette.push_back(ez_color(0x4c, 0x26, 0x00));
    palette.push_back(ez_color(0x4c, 0x39, 0x26));
    palette.push_back(ez_color(0x26, 0x13, 0x00));
    palette.push_back(ez_color(0x26, 0x1c, 0x13));
    palette.push_back(ez_color(0xff, 0xbf, 0x00));
    palette.push_back(ez_color(0xff, 0xdf, 0x7f));
    palette.push_back(ez_color(0xa5, 0x7c, 0x00));
    palette.push_back(ez_color(0xa5, 0x91, 0x52));
    palette.push_back(ez_color(0x7f, 0x5f, 0x00));
    palette.push_back(ez_color(0x7f, 0x6f, 0x3f));
    palette.push_back(ez_color(0x4c, 0x39, 0x00));
    palette.push_back(ez_color(0x4c, 0x42, 0x26));
    palette.push_back(ez_color(0x26, 0x1c, 0x00));
    palette.push_back(ez_color(0x26, 0x21, 0x13));
    palette.push_back(ez_color(0xff, 0xff, 0x00));
    palette.push_back(ez_color(0xff, 0xff, 0x7f));
    palette.push_back(ez_color(0xa5, 0xa5, 0x00));
    palette.push_back(ez_color(0xa5, 0xa5, 0x52));
    palette.push_back(ez_color(0x7f, 0x7f, 0x00));
    palette.push_back(ez_color(0x7f, 0x7f, 0x3f));
    palette.push_back(ez_color(0x4c, 0x4c, 0x00));
    palette.push_back(ez_color(0x4c, 0x4c, 0x26));
    palette.push_back(ez_color(0x26, 0x26, 0x00));
    palette.push_back(ez_color(0x26, 0x26, 0x13));
    palette.push_back(ez_color(0xbf, 0xff, 0x00));
    palette.push_back(ez_color(0xdf, 0xff, 0x7f));
    palette.push_back(ez_color(0x7c, 0xa5, 0x00));
    palette.push_back(ez_color(0x91, 0xa5, 0x52));
    palette.push_back(ez_color(0x5f, 0x7f, 0x00));
    palette.push_back(ez_color(0x6f, 0x7f, 0x3f));
    palette.push_back(ez_color(0x39, 0x4c, 0x00));
    palette.push_back(ez_color(0x42, 0x4c, 0x26));
    palette.push_back(ez_color(0x1c, 0x26, 0x00));
    palette.push_back(ez_color(0x21, 0x26, 0x13));
    palette.push_back(ez_color(0x7f, 0xff, 0x00));
    palette.push_back(ez_color(0xbf, 0xff, 0x7f));
    palette.push_back(ez_color(0x52, 0xa5, 0x00));
    palette.push_back(ez_color(0x7c, 0xa5, 0x52));
    palette.push_back(ez_color(0x3f, 0x7f, 0x00));
    palette.push_back(ez_color(0x5f, 0x7f, 0x3f));
    palette.push_back(ez_color(0x26, 0x4c, 0x00));
    palette.push_back(ez_color(0x39, 0x4c, 0x26));
    palette.push_back(ez_color(0x13, 0x26, 0x00));
    palette.push_back(ez_color(0x1c, 0x26, 0x13));
    palette.push_back(ez_color(0x3f, 0xff, 0x00));
    palette.push_back(ez_color(0x9f, 0xff, 0x7f));
    palette.push_back(ez_color(0x29, 0xa5, 0x00));
    palette.push_back(ez_color(0x67, 0xa5, 0x52));
    palette.push_back(ez_color(0x1f, 0x7f, 0x00));
    palette.push_back(ez_color(0x4f, 0x7f, 0x3f));
    palette.push_back(ez_color(0x13, 0x4c, 0x00));
    palette.push_back(ez_color(0x2f, 0x4c, 0x26));
    palette.push_back(ez_color(0x09, 0x26, 0x00));
    palette.push_back(ez_color(0x17, 0x26, 0x13));
    palette.push_back(ez_color(0x00, 0xff, 0x00));
    palette.push_back(ez_color(0x7f, 0xff, 0x7f));
    palette.push_back(ez_color(0x00, 0xa5, 0x00));
    palette.push_back(ez_color(0x52, 0xa5, 0x52));
    palette.push_back(ez_color(0x00, 0x7f, 0x00));
    palette.push_back(ez_color(0x3f, 0x7f, 0x3f));
    palette.push_back(ez_color(0x00, 0x4c, 0x00));
    palette.push_back(ez_color(0x26, 0x4c, 0x26));
    palette.push_back(ez_color(0x00, 0x26, 0x00));
    palette.push_back(ez_color(0x13, 0x26, 0x13));
    palette.push_back(ez_color(0x00, 0xff, 0x3f));
    palette.push_back(ez_color(0x7f, 0xff, 0x9f));
    palette.push_back(ez_color(0x00, 0xa5, 0x29));
    palette.push_back(ez_color(0x52, 0xa5, 0x67));
    palette.push_back(ez_color(0x00, 0x7f, 0x1f));
    palette.push_back(ez_color(0x3f, 0x7f, 0x4f));
    palette.push_back(ez_color(0x00, 0x4c, 0x13));
    palette.push_back(ez_color(0x26, 0x4c, 0x2f));
    palette.push_back(ez_color(0x00, 0x26, 0x09));
    palette.push_back(ez_color(0x13, 0x26, 0x17));
    palette.push_back(ez_color(0x00, 0xff, 0x7f));
    palette.push_back(ez_color(0x7f, 0xff, 0xbf));
    palette.push_back(ez_color(0x00, 0xa5, 0x52));
    palette.push_back(ez_color(0x52, 0xa5, 0x7c));
    palette.push_back(ez_color(0x00, 0x7f, 0x3f));
    palette.push_back(ez_color(0x3f, 0x7f, 0x5f));
    palette.push_back(ez_color(0x00, 0x4c, 0x26));
    palette.push_back(ez_color(0x26, 0x4c, 0x39));
    palette.push_back(ez_color(0x00, 0x26, 0x13));
    palette.push_back(ez_color(0x13, 0x26, 0x1c));
    palette.push_back(ez_color(0x00, 0xff, 0xbf));
    palette.push_back(ez_color(0x7f, 0xff, 0xdf));
    palette.push_back(ez_color(0x00, 0xa5, 0x7c));
    palette.push_back(ez_color(0x52, 0xa5, 0x91));
    palette.push_back(ez_color(0x00, 0x7f, 0x5f));
    palette.push_back(ez_color(0x3f, 0x7f, 0x6f));
    palette.push_back(ez_color(0x00, 0x4c, 0x39));
    palette.push_back(ez_color(0x26, 0x4c, 0x42));
    palette.push_back(ez_color(0x00, 0x26, 0x1c));
    palette.push_back(ez_color(0x13, 0x26, 0x21));
    palette.push_back(ez_color(0x00, 0xff, 0xff));
    palette.push_back(ez_color(0x7f, 0xff, 0xff));
    palette.push_back(ez_color(0x00, 0xa5, 0xa5));
    palette.push_back(ez_color(0x52, 0xa5, 0xa5));
    palette.push_back(ez_color(0x00, 0x7f, 0x7f));
    palette.push_back(ez_color(0x3f, 0x7f, 0x7f));
    palette.push_back(ez_color(0x00, 0x4c, 0x4c));
    palette.push_back(ez_color(0x26, 0x4c, 0x4c));
    palette.push_back(ez_color(0x00, 0x26, 0x26));
    palette.push_back(ez_color(0x13, 0x26, 0x26));
    palette.push_back(ez_color(0x00, 0xbf, 0xff));
    palette.push_back(ez_color(0x7f, 0xdf, 0xff));
    palette.push_back(ez_color(0x00, 0x7c, 0xa5));
    palette.push_back(ez_color(0x52, 0x91, 0xa5));
    palette.push_back(ez_color(0x00, 0x5f, 0x7f));
    palette.push_back(ez_color(0x3f, 0x6f, 0x7f));
    palette.push_back(ez_color(0x00, 0x39, 0x4c));
    palette.push_back(ez_color(0x26, 0x42, 0x4c));
    palette.push_back(ez_color(0x00, 0x1c, 0x26));
    palette.push_back(ez_color(0x13, 0x21, 0x26));
    palette.push_back(ez_color(0x00, 0x7f, 0xff));
    palette.push_back(ez_color(0x7f, 0xbf, 0xff));
    palette.push_back(ez_color(0x00, 0x52, 0xa5));
    palette.push_back(ez_color(0x52, 0x7c, 0xa5));
    palette.push_back(ez_color(0x00, 0x3f, 0x7f));
    palette.push_back(ez_color(0x3f, 0x5f, 0x7f));
    palette.push_back(ez_color(0x00, 0x26, 0x4c));
    palette.push_back(ez_color(0x26, 0x39, 0x4c));
    palette.push_back(ez_color(0x00, 0x13, 0x26));
    palette.push_back(ez_color(0x13, 0x1c, 0x26));
    palette.push_back(ez_color(0x00, 0x3f, 0xff));
    palette.push_back(ez_color(0x7f, 0x9f, 0xff));
    palette.push_back(ez_color(0x00, 0x29, 0xa5));
    palette.push_back(ez_color(0x52, 0x67, 0xa5));
    palette.push_back(ez_color(0x00, 0x1f, 0x7f));
    palette.push_back(ez_color(0x3f, 0x4f, 0x7f));
    palette.push_back(ez_color(0x00, 0x13, 0x4c));
    palette.push_back(ez_color(0x26, 0x2f, 0x4c));
    palette.push_back(ez_color(0x00, 0x09, 0x26));
    palette.push_back(ez_color(0x13, 0x17, 0x26));
    palette.push_back(ez_color(0x00, 0x00, 0xff));
    palette.push_back(ez_color(0x7f, 0x7f, 0xff));
    palette.push_back(ez_color(0x00, 0x00, 0xa5));
    palette.push_back(ez_color(0x52, 0x52, 0xa5));
    palette.push_back(ez_color(0x00, 0x00, 0x7f));
    palette.push_back(ez_color(0x3f, 0x3f, 0x7f));
    palette.push_back(ez_color(0x00, 0x00, 0x4c));
    palette.push_back(ez_color(0x26, 0x26, 0x4c));
    palette.push_back(ez_color(0x00, 0x00, 0x26));
    palette.push_back(ez_color(0x13, 0x13, 0x26));
    palette.push_back(ez_color(0x3f, 0x00, 0xff));
    palette.push_back(ez_color(0x9f, 0x7f, 0xff));
    palette.push_back(ez_color(0x29, 0x00, 0xa5));
    palette.push_back(ez_color(0x67, 0x52, 0xa5));
    palette.push_back(ez_color(0x1f, 0x00, 0x7f));
    palette.push_back(ez_color(0x4f, 0x3f, 0x7f));
    palette.push_back(ez_color(0x13, 0x00, 0x4c));
    palette.push_back(ez_color(0x2f, 0x26, 0x4c));
    palette.push_back(ez_color(0x09, 0x00, 0x26));
    palette.push_back(ez_color(0x17, 0x13, 0x26));
    palette.push_back(ez_color(0x7f, 0x00, 0xff));
    palette.push_back(ez_color(0xbf, 0x7f, 0xff));
    palette.push_back(ez_color(0x52, 0x00, 0xa5));
    palette.push_back(ez_color(0x7c, 0x52, 0xa5));
    palette.push_back(ez_color(0x3f, 0x00, 0x7f));
    palette.push_back(ez_color(0x5f, 0x3f, 0x7f));
    palette.push_back(ez_color(0x26, 0x00, 0x4c));
    palette.push_back(ez_color(0x39, 0x26, 0x4c));
    palette.push_back(ez_color(0x13, 0x00, 0x26));
    palette.push_back(ez_color(0x1c, 0x13, 0x26));
    palette.push_back(ez_color(0xbf, 0x00, 0xff));
    palette.push_back(ez_color(0xdf, 0x7f, 0xff));
    palette.push_back(ez_color(0x7c, 0x00, 0xa5));
    palette.push_back(ez_color(0x91, 0x52, 0xa5));
    palette.push_back(ez_color(0x5f, 0x00, 0x7f));
    palette.push_back(ez_color(0x6f, 0x3f, 0x7f));
    palette.push_back(ez_color(0x39, 0x00, 0x4c));
    palette.push_back(ez_color(0x42, 0x26, 0x4c));
    palette.push_back(ez_color(0x1c, 0x00, 0x26));
    palette.push_back(ez_color(0x21, 0x13, 0x26));
    palette.push_back(ez_color(0xff, 0x00, 0xff));
    palette.push_back(ez_color(0xff, 0x7f, 0xff));
    palette.push_back(ez_color(0xa5, 0x00, 0xa5));
    palette.push_back(ez_color(0xa5, 0x52, 0xa5));
    palette.push_back(ez_color(0x7f, 0x00, 0x7f));
    palette.push_back(ez_color(0x7f, 0x3f, 0x7f));
    palette.push_back(ez_color(0x4c, 0x00, 0x4c));
    palette.push_back(ez_color(0x4c, 0x26, 0x4c));
    palette.push_back(ez_color(0x26, 0x00, 0x26));
    palette.push_back(ez_color(0x26, 0x13, 0x26));
    palette.push_back(ez_color(0xff, 0x00, 0xbf));
    palette.push_back(ez_color(0xff, 0x7f, 0xdf));
    palette.push_back(ez_color(0xa5, 0x00, 0x7c));
    palette.push_back(ez_color(0xa5, 0x52, 0x91));
    palette.push_back(ez_color(0x7f, 0x00, 0x5f));
    palette.push_back(ez_color(0x7f, 0x3f, 0x6f));
    palette.push_back(ez_color(0x4c, 0x00, 0x39));
    palette.push_back(ez_color(0x4c, 0x26, 0x42));
    palette.push_back(ez_color(0x26, 0x00, 0x1c));
    palette.push_back(ez_color(0x26, 0x13, 0x21));
    palette.push_back(ez_color(0xff, 0x00, 0x7f));
    palette.push_back(ez_color(0xff, 0x7f, 0xbf));
    palette.push_back(ez_color(0xa5, 0x00, 0x52));
    palette.push_back(ez_color(0xa5, 0x52, 0x7c));
    palette.push_back(ez_color(0x7f, 0x00, 0x3f));
    palette.push_back(ez_color(0x7f, 0x3f, 0x5f));
    palette.push_back(ez_color(0x4c, 0x00, 0x26));
    palette.push_back(ez_color(0x4c, 0x26, 0x39));
    palette.push_back(ez_color(0x26, 0x00, 0x13));
    palette.push_back(ez_color(0x26, 0x13, 0x1c));
    palette.push_back(ez_color(0xff, 0x00, 0x3f));
    palette.push_back(ez_color(0xff, 0x7f, 0x9f));
    palette.push_back(ez_color(0xa5, 0x00, 0x29));
    palette.push_back(ez_color(0xa5, 0x52, 0x67));
    palette.push_back(ez_color(0x7f, 0x00, 0x1f));
    palette.push_back(ez_color(0x7f, 0x3f, 0x4f));
    palette.push_back(ez_color(0x4c, 0x00, 0x13));
    palette.push_back(ez_color(0x4c, 0x26, 0x2f));
    palette.push_back(ez_color(0x26, 0x00, 0x09));
    palette.push_back(ez_color(0x26, 0x13, 0x17));
    palette.push_back(ez_color(0x54, 0x54, 0x54));
    palette.push_back(ez_color(0x76, 0x76, 0x76));
    palette.push_back(ez_color(0x98, 0x98, 0x98));
    palette.push_back(ez_color(0xba, 0xba, 0xba));
    palette.push_back(ez_color(0xdc, 0xdc, 0xdc));
    palette.push_back(ez_color(0xff, 0xff, 0xff));
    palette.white = palette.number_of_colors() - 1;  //white
    palette.last  = palette.number_of_colors() - 1;  //last
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "ezfbHues";
    palette.push_back(ez_color(0x00, 0x00, 0x00)); // black
    palette.black = 0;
    palette.first = 1;
    for(i = 1; i < EZ_PALETTE_MAX - 1; i++)
    {
        hue = (i - 1) * 1530 / (EZ_PALETTE_MAX - 1);
        shift = hue % 255;
        if(hue < 255)
        {
            r = 255;
            g = shift;
        }
        else if(hue < 510)
        {
            r = 256 - shift;
            g = 255;
        }
        else if(hue < 765)
        {
            g = 255;
            b = shift;
        }
        else if(hue < 1020)
        {
            g = 256 - shift;
            b = 255;
        }
        else if(hue < 1275)
        {
            r = shift;
            b = 255;
        }
        else
        {
            r = 255;
            b = 256 - shift;
        }
        palette.push_back(ez_color(r, g, b));
    }
    palette.gray  = 128;
    palette.last  = 255;
    palette.push_back(ez_color(0xff, 0xff, 0xff)); // white
    palette.white = 255;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "RedGreen";
    palette.push_back(ez_color(0x00, 0x00, 0x00)); // black
    palette.black = 0;
    palette.first = 1;
    for(i = 1; i < EZ_PALETTE_MAX; i++)
    {
        hue = (i - 1) * 510 / (EZ_PALETTE_MAX - 1);
        shift = hue % 255;
        if(hue < 255)
        {
            r = 255;
            g = shift;
        }
        else if(hue < 510)
        {
            r = 253 - shift;
            g = 255;
        }
        palette.push_back(ez_color(r, g, 0));
    }
    palette.gray  = 128;
    palette.last  = 255;
    palette.white = 255;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "ezfb_332";
    for(i = 0; i < EZ_PALETTE_MAX; i++)
        palette.push_back(ez_color(  i & 0xe0,
                                      (i & 0x1c) << 3,
                                      (i & 0x03) << 6
                                    )
                         );
    palette.first =   1;
    palette.last  = 255;
    palette.white = 255;
    palette.gray  = 128;
    palette.black =   0;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "ezfb_323";
    for(i = 0; i < EZ_PALETTE_MAX; i++)
        palette.push_back(ez_color(  i & 0xe0,
                                      (i & 0x18) << 3,
                                      (i & 0x07) << 6
                                    )
                         );
    palette.first =   1;
    palette.last  = 255;
    palette.white = 255;
    palette.gray  = 128;
    palette.black =   0;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "ezfb_233";
    for(i = 0; i < EZ_PALETTE_MAX; i++)
        palette.push_back(ez_color(  i & 0xc0,
                                      (i & 0x38) << 3,
                                      (i & 0x07) << 6
                                    )
                         );
    palette.first =   1;
    palette.last  = 255;
    palette.white = 255;
    palette.gray  = 128;
    palette.black =   0;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "Reds____";
    for(i = 0; i < EZ_PALETTE_MAX; i++)
        palette.push_back(ez_color(255 - i, 0, 0));
    palette.first =   0;
    palette.last  = 255;
    palette.white =   0;
    palette.gray  = 128;
    palette.black = 255;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "Greens__";
    for(i = 0; i < EZ_PALETTE_MAX; i++)
        palette.push_back(ez_color(0, 255 - i, 0));
    palette.first =   1;
    palette.last  = 255;
    palette.white = 255;
    palette.gray  = 128;
    palette.black =   0;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "Blues___";
    for(i = 0; i < EZ_PALETTE_MAX; i++)
        palette.push_back(ez_color(0, 0, 255 - i));
    palette.first =   0;
    palette.last  = 255;
    palette.white =   0;
    palette.gray  = 128;
    palette.black = 255;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "Yellows_";
    for(i = 0; i < EZ_PALETTE_MAX; i++)
        palette.push_back(ez_color(255 - i, 255 - i, 0));
    palette.first =   0;
    palette.last  = 255;
    palette.white =   0;
    palette.gray  = 128;
    palette.black = 255;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "Magentas";
    for(i = 0; i < EZ_PALETTE_MAX; i++)
        palette.push_back(ez_color(255 - i, 0, 255 - i));
    palette.first =   0;
    palette.last  = 255;
    palette.white =   0;
    palette.gray  = 128;
    palette.black = 255;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "Cyans___";
    for(i = 0; i < EZ_PALETTE_MAX; i++)
        palette.push_back(ez_color(0, 255 - i, 255 - i));
    palette.first =   0;
    palette.last  = 255;
    palette.white =   0;
    palette.gray  = 128;
    palette.black = 255;
    push_back(palette);
    palette.clear();
    //------------------------------------------------------------------------
    palette.name = "Grays___";
    for(i = 0; i < EZ_PALETTE_MAX; i++)
        palette.push_back(ez_color(255 - i, 255 - i, 255 - i));
    palette.first =   0;
    palette.last  = 255;
    palette.white =   0;
    palette.gray  = 128;
    palette.black = 255;
    push_back(palette);
    //------------------------------------------------------------------------
    return;
}

//############################################################################
void ez_palette_set::display_stats() const
{    
    int i,
        y_line = 8 * ez_ilda_pointer->font_size_factor,
        x_off  = BMP.yres + EZ_STATS_DISPLAY_OFFSET * ez_ilda_pointer->font_size_factor,
        y_off  = EZ_STATS_DISPLAY_OFFSET * ez_ilda_pointer->font_size_factor;
    
    bmp_printf(   &BMP
                , x_off
                , 0
                , BMP.white
                , BMP.black
                , 0
                , ez_ilda_pointer->font_size_factor
                , "                  "
              );
    bmp_printf(   &BMP
                , x_off
                , y_off
                , BMP.white
                , BMP.black
                , 0
                , ez_ilda_pointer->font_size_factor
                , "Palette set stats "
              );
    y_off += y_line;
    bmp_printf(   &BMP
                , x_off
                , y_off
                , next_hue()
                , BMP.black
                , 0
                , ez_ilda_pointer->font_size_factor
                , "total palettes %d "
                , number_of_palettes()
              );
    y_off += y_line;
    bmp_printf(   &BMP
                , x_off
                , y_off
                , next_hue()
                , BMP.black
                , 0
                , ez_ilda_pointer->font_size_factor
                , "added palettes %d"
                , ez_ilda_pointer->number_of_palettes() - EZ_BUILT_IN_PALETTES
              );
    for(i = 0; i < number_of_palettes(); i++)
    {
        y_off += y_line;
        bmp_printf(   &BMP
                    , x_off
                    , y_off
                    , next_hue()
                    , BMP.black
                    , 0
                    , ez_ilda_pointer->font_size_factor
                    , "%s          "
                    , at(i).name.c_str()
                  );
    }
    bmp_to_SDL_Surface(&BMP, screen);
    return;
}

//############################################################################
void ez_palette_set::set_selected_palette_color(u_char c)
{
    if(c >= ez_ilda_pointer->current_palette().number_of_colors())
        c = 0;
    selected_palette_color = c;
    return;
}

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################
