//###################################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//###################################################################################
#ifndef __EZ_3D_REAL_DEFINITIONS__
#define __EZ_3D_REAL_DEFINITIONS__

//###################################################################################
#include "ez_ilda_common.hpp"
#include "ez_color.hpp"
#include "ez_3D_float.hpp"

//###################################################################################
class ez_real_point : public ez_3D_float, public ez_color
{
public:
    ez_real_point( const float&   _x = 0,
                   const float&   _y = 0,
                   const float&   _z = 0,
                   const u_char&  _r = 0,
                   const u_char&  _g = 0,
                   const u_char&  _b = 0,
                   const u_char&  _k = EZ_ILDA_BLANKING_BIT,
                   const u_char&  _c = 0
                  )
                  : ez_3D_float (_x, _y, _z),
                    ez_color    (_r, _g, _b),
                    k           (_k        ),
                    c           (_c        )                   {}
    //-------------------------------------------------------------------------------
    ez_real_point(const ez_real_point& p)
                 : ez_3D_float (p.x, p.y, p.z),
                   ez_color    (p.r, p.g, p.b),
                   k           (p.k          ),
                   c           (p.c          )                 {}
    //-------------------------------------------------------------------------------
    ez_real_point( const ez_3D_float& f,
                   const ez_color&  rgb,
                   const u_char&       _k = EZ_ILDA_BLANKING_BIT,
                   const u_char&       _c = 0
                 )
                 : ez_3D_float (f  ),
                   ez_color    (rgb),
                   k           (_k ),
                   c           (_c )                           {}
    //-------------------------------------------------------------------------------
    ez_real_point( const ez_3D_float& f,
                   const u_char&      _c
                 )
                 : ez_3D_float (f  ),
                   ez_color    (0, 0, 0),
                   k           (EZ_ILDA_BLANKING_BIT),
                   c           (_c )                           {}
    //-------------------------------------------------------------------------------
    ez_real_point( const ez_3D_float& f)
                 : ez_3D_float (f                ),
                   ez_color    (0, 0, 0          ),
                   k           (EZ_ILDA_BLANKING_BIT),
                   c           (0                )             {}
    //-------------------------------------------------------------------------------
    ez_real_point( const ez_color& rgb)
                 : ez_3D_float (0, 0, 0          ),
                   ez_color    (rgb              ),
                   k           (EZ_ILDA_BLANKING_BIT),
                   c           (0                )             {}
    //-------------------------------------------------------------------------------
    inline ez_real_point& operator = (const ez_real_point& p)
                                {
                                    x = p.x;
                                    y = p.y;
                                    z = p.z;
                                    r = p.r;
                                    g = p.g;
                                    b = p.b;
                                    k = p.k;
                                    c = p.c;
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_point& operator = (const ez_3D_float& f) // only assign the coordinates
                                {
                                    x = f.x;
                                    y = f.y;
                                    z = f.z;
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_point  operator + (const ez_3D_float& f)
                                {
                                    ez_real_point sum(*this);
                                    sum.x += f.x;
                                    sum.y += f.y;
                                    sum.z += f.z;
                                    sum.k =  k;
                                    sum.c =  c;
                                    sum.r =  r;
                                    sum.g =  g;
                                    sum.b =  b;
                                    return sum;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_point& operator += (const ez_3D_float& f)
                                {
                                    x += f.x;
                                    y += f.y;
                                    z += f.z;
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_point  operator - (const ez_3D_float& f)
                                {
                                    ez_real_point diff(*this);
                                    diff.x -= f.x;
                                    diff.y -= f.y;
                                    diff.z -= f.z;
                                    return diff;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_point& operator -= (const ez_3D_float& f)
                                {
                                    x -= f.x;
                                    y -= f.y;
                                    z -= f.z;
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_point& operator *= (const ez_3D_float& f)
                                {
                                    x *= f.x;
                                    y *= f.y;
                                    z *= f.z;
                                    return *this;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_point operator * (const ez_3D_float& f)
                                {
                                    ez_real_point product;
                                    product.r = r;
                                    product.g = g;
                                    product.b = b;
                                    product.x = x * f.x;
                                    product.y = y * f.y;
                                    product.z = z * f.z;
                                    product.k = k;
                                    product.c = c;
                                    return product;
                                }
    //-------------------------------------------------------------------------------
    inline bool operator == (const ez_real_point& p) const
                        {   
                            if(    (x != p.x)
                                || (y != p.y)
                                || (z != p.z)
                                || (r != p.r)
                                || (g != p.g)
                                || (b != p.b)
                                || ((k & EZ_ILDA_BLANKING_BIT) != (p.k & EZ_ILDA_BLANKING_BIT))
                                || (c != p.c)
                              )
                                return false;
                            return true;
                        }
    //-------------------------------------------------------------------------------
    inline bool operator != (const ez_real_point& p) const
                        {                               
                            if(    (x == p.x)
                                && (y == p.y)
                                && (z == p.z)
                                && (r == p.r)
                                && (g == p.g)
                                && (b == p.b)
                                && ((k & EZ_ILDA_BLANKING_BIT) == (p.k & EZ_ILDA_BLANKING_BIT))
                                && (c == p.c)
                              )
                                return false;
                            return true;
                        }
    //-------------------------------------------------------------------------------
    inline ez_real_point operator - ()
                                {
                                    ez_real_point negative;
                                    negative.x = -x;
                                    negative.y = -y;
                                    negative.z = -z;
                                    return negative;
                                }
    //-------------------------------------------------------------------------------
    inline ez_real_point& blank   ()   {    k |=  EZ_ILDA_BLANKING_BIT; return *this;   }
    inline ez_real_point& unblank ()   {    k &= ~EZ_ILDA_BLANKING_BIT; return *this;   }
    //-------------------------------------------------------------------------------
    inline bool  is_blank     () const {    return (bool)(k & EZ_ILDA_BLANKING_BIT);    }
    inline bool  is_lit       () const {    return !is_blank()                     ;    }
    //-------------------------------------------------------------------------------
    inline void  set_unique   ()       {    k |=  EZ_ILDA_UNIQUE_POINT;      return;    }
    inline void  clear_unique ()       {    k &= ~EZ_ILDA_UNIQUE_POINT;      return;    }
    inline bool  is_unique    () const {    return (bool)(k & EZ_ILDA_UNIQUE_POINT);    }
    //-------------------------------------------------------------------------------
    inline bool  is_black () const
                    {
                        return (bool)(    is_lit()
                                       && r == 0
                                       && g == 0
                                       && b == 0
                                     );
                    }
    //-------------------------------------------------------------------------------
    inline ez_3D_float position() const    {    return (ez_3D_float)(*this);    }
    //-------------------------------------------------------------------------------
    inline int color() const
                {
                    return   is_lit()
                           ? (int)(   r << 16
                                    | g <<  8
                                    | b
                                  )
                           :  0;
                }
    //-------------------------------------------------------------------------------
    u_char  k, c;
};

//###################################################################################
#endif

//###################################################################################
/////////////////////////////////////////////////////////////////////////////////////
//###################################################################################



