//############################################################################
// ~ International Laser Display Association ~
// ~     ILDA file format driver in C++      ~
// ~             built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//############################################################################
#include "ez_wave.hpp"
    
//############################################################################
ez_wave_header::ez_wave_header( const int    s,
                                const int    r,
                                const short  n,
                                const short  b
                              )
                                : num_samples     (s),
                                  sample_rate     (r),
                                  ez_wave_mode    (   EZ_WAVE_POLARITY
                                                    | EZ_WAVE_END_OF_FRAME
                                                    | EZ_WAVE_UNIQUE_FRAME
                                                    | EZ_WAVE_UNIQUE_POINT
                                                  )  ,
                                  num_channels    (n),
                                  bits_per_sample (b)
{
    offset = new int[num_channels];
    for(int i = 0; i < num_channels; i++)
        offset[i] = 0;
}

//############################################################################
ez_wave_header::~ez_wave_header()
{
    delete offset;
}

//############################################################################
ez_wave_header::ez_wave_header(const ez_wave_header& header)
                                : num_samples     (header.num_samples    ),
                                  sample_rate     (header.sample_rate    ),
                                  ez_wave_mode    (header.ez_wave_mode   ),
                                  num_channels    (header.num_channels   ),
                                  bits_per_sample (header.bits_per_sample)
{
    offset = new int[num_channels];
    memcpy((void*)offset, (void*)header.offset, num_channels * sizeof(int));
}

//############################################################################
ez_wave_header& ez_wave_header::operator = (const ez_wave_header& header)
{
    delete offset;
    num_samples     = header.num_samples    ;
    sample_rate     = header.sample_rate    ;
    ez_wave_mode    = header.ez_wave_mode   ;
    num_channels    = header.num_channels   ;
    bits_per_sample = header.bits_per_sample;
    offset          = new int[num_channels] ;
    memcpy((void*)offset, (void*)header.offset, num_channels * sizeof(int));
    return *this;
}

//############################################################################
int ez_wave_header::from_ifstream(ifstream& in)
{
    char  a, b, c, d;

    int   i                ,
          chunk_size       ,
          sub_chunk_1_size ,
          sub_chunk_2_size ,
          byte_rate        ;

    short audio_format ,
          block_align  ;

    if(!in.eof())
    {
        in.get(a); if(a != 'R') return EZ_ILDA_BAD_HEADER;
        in.get(a); if(a != 'I') return EZ_ILDA_BAD_HEADER;
        in.get(a); if(a != 'F') return EZ_ILDA_BAD_HEADER;
        in.get(a); if(a != 'F') return EZ_ILDA_BAD_HEADER;
#if 0 //EZ_ILDA_DEBUG_MESSAGES
        cout << "\nRIFF             : ";
#endif
        //--------------------------------------------------------------------
        in.get(a); in.get(b); in.get(c); in.get(d);
        chunk_size = (   (a & 0x000000ff)
                       | (b & 0x000000ff) << 8 
                       | (c & 0x000000ff) << 16
                       | (d & 0x000000ff) << 24
                     );
#if 0 //EZ_ILDA_DEBUG_MESSAGES
        cout << "\nchunk_size       : " << chunk_size;
#endif
        //--------------------------------------------------------------------
        in.get(a); if(a != 'W') return EZ_ILDA_BAD_HEADER;
        in.get(a); if(a != 'A') return EZ_ILDA_BAD_HEADER;
        in.get(a); if(a != 'V') return EZ_ILDA_BAD_HEADER;
        in.get(a); if(a != 'E') return EZ_ILDA_BAD_HEADER;
        //--------------------------------------------------------------------
        in.get(a); if(a != 'f') return EZ_ILDA_BAD_HEADER;
        in.get(a); if(a != 'm') return EZ_ILDA_BAD_HEADER;
        in.get(a); if(a != 't') return EZ_ILDA_BAD_HEADER;
        in.get(a); if(a != ' ') return EZ_ILDA_BAD_HEADER;
#if 0 //EZ_ILDA_DEBUG_MESSAGES
        cout << "\nWAVEfmt          : ";
#endif
        //--------------------------------------------------------------------
        in.get(a); in.get(b); in.get(c); in.get(d);
        sub_chunk_1_size = (   (a & 0x000000ff)
                             | (b & 0x000000ff) << 8 
                             | (c & 0x000000ff) << 16
                             | (d & 0x000000ff) << 24
                           );
#if 0 //EZ_ILDA_DEBUG_MESSAGES
        cout << "\nsub_chunk_1_size : " << sub_chunk_1_size;
#endif
        //--------------------------------------------------------------------
        in.get(a); in.get(b);
        audio_format = ((b & 0x00ff) << 8 | (a & 0x00ff));  
#if 0 //EZ_ILDA_DEBUG_MESSAGES
        cout << "\naudio_format     : " << audio_format;
#endif
        if(audio_format != 1) return EZ_ILDA_INCOMPATIBLE_FORMATS;
        //--------------------------------------------------------------------
        in.get(a); in.get(b);
        num_channels = ((b & 0x00ff) << 8 | (a & 0x00ff));
#if EZ_ILDA_DEBUG_MESSAGES
        cout << "\nnum_channels     : " << num_channels;
#endif
        //--------------------------------------------------------------------
        in.get(a); in.get(b); in.get(c); in.get(d);
        sample_rate = (   (a & 0x000000ff)
                        | (b & 0x000000ff) << 8 
                        | (c & 0x000000ff) << 16
                        | (d & 0x000000ff) << 24
                      );
#if EZ_ILDA_DEBUG_MESSAGES
        cout << "\nsample_rate      : " << sample_rate;
#endif
        //--------------------------------------------------------------------
        in.get(a); in.get(b); in.get(c); in.get(d);
        byte_rate = (   (a & 0x000000ff)
                      | (b & 0x000000ff) << 8 
                      | (c & 0x000000ff) << 16
                      | (d & 0x000000ff) << 24
                    );
#if 0 //EZ_ILDA_DEBUG_MESSAGES
        cout << "\nbyte_rate        : " << byte_rate;
#endif
        //--------------------------------------------------------------------
        in.get(a); in.get(b);
        block_align = ((b & 0x00ff) << 8 | (a & 0x00ff));
#if 0 //EZ_ILDA_DEBUG_MESSAGES
        cout << "\nblock_align      : " << block_align;
#endif
        //--------------------------------------------------------------------
        in.get(a); in.get(b);
        bits_per_sample = ((b & 0x00ff) << 8 | (a & 0x00ff));
#if EZ_ILDA_DEBUG_MESSAGES
        cout << "\nbits_per_sample  : " << bits_per_sample;
#endif
        //--------------------------------------------------------------------
        if(byte_rate   != (sample_rate * num_channels * (bits_per_sample / 8)))
            return EZ_ILDA_BAD_HEADER;
        if(block_align != (num_channels * (bits_per_sample / 8)))
            return EZ_ILDA_BAD_HEADER;
        //--------------------------------------------------------------------
        if(sub_chunk_1_size > 16) // We might have sample offset data
        {
            in.get(a); if(a != 'o') return EZ_ILDA_BAD_HEADER;
            in.get(a); if(a != 'f') return EZ_ILDA_BAD_HEADER;
            in.get(a); if(a != 'f') return EZ_ILDA_BAD_HEADER;
            in.get(a); if(a != 's') return EZ_ILDA_BAD_HEADER;
            in.get(a); if(a != 'e') return EZ_ILDA_BAD_HEADER;
            in.get(a); if(a != 't') return EZ_ILDA_BAD_HEADER;
            in.get(a); if(a != 's') return EZ_ILDA_BAD_HEADER;
            in.get(a); if(a != ' ') return EZ_ILDA_BAD_HEADER;
#if EZ_ILDA_DEBUG_MESSAGES
            cout << "\noffsets          : ";
#endif
            //----------------------------------------------------------------
            delete offset;
            offset = new int[num_channels];
            for(i = 0; i < num_channels; i++)
            {
                in.get(a); in.get(b); in.get(c); in.get(d);
                offset[i] = (   (a & 0x000000ff)
                              | (b & 0x000000ff) << 8 
                              | (c & 0x000000ff) << 16
                              | (d & 0x000000ff) << 24
                            );
#if EZ_ILDA_DEBUG_MESSAGES
                cout << "\noffset[" << i << "]        : " << offset[i];
#endif
            }
            //----------------------------------------------------------------
            in.get(a); in.get(b); in.get(c); in.get(d);
            if(    (a == 'e') // we may or may not have an "eof3" mark
                && (b == 'o')
                && (c == 'f')
                && (d == '3')
              )
            {
//#if EZ_ILDA_DEBUG_MESSAGES
                cout << "\neof3            : ";
//#endif
            }
            else
                ez_wave_mode = (   (a & 0x000000ff)
                                 | (b & 0x000000ff) << 8 
                                 | (c & 0x000000ff) << 16
                                 | (d & 0x000000ff) << 24
                               );
            //----------------------------------------------------------------
        }
        in.get(a); in.get(b); in.get(c); in.get(d);
        //--------------------------------------------------------------------
        if(a != 'd') return EZ_ILDA_BAD_HEADER;
        if(b != 'a') return EZ_ILDA_BAD_HEADER;
        if(c != 't') return EZ_ILDA_BAD_HEADER;
        if(d != 'a') return EZ_ILDA_BAD_HEADER;
#if 0 //EZ_ILDA_DEBUG_MESSAGES
        cout << "\ndata             : ";
#endif
        //--------------------------------------------------------------------
        in.get(a); in.get(b); in.get(c); in.get(d);
        sub_chunk_2_size = (   (a & 0x000000ff)
                             | (b & 0x000000ff) << 8 
                             | (c & 0x000000ff) << 16
                             | (d & 0x000000ff) << 24
                           );
#if 0 //EZ_ILDA_DEBUG_MESSAGES
        cout << "\nsub_chunk_2_size : " << sub_chunk_2_size;
#endif
        //if(sub_chunk_2_size != (chunk_size + 36)) return EZ_ILDA_BAD_HEADER;
        //--------------------------------------------------------------------
        num_samples = sub_chunk_2_size / (num_channels * (bits_per_sample / 8));
#if EZ_ILDA_DEBUG_MESSAGES
        cout << "\nnum_samples      : " << num_samples;
        cout << "\nlength in seconds: " << (double)num_samples / (double)sample_rate << endl << endl;
#endif
    }
    return in.bad();
}

//############################################################################
void ez_wave_header::to_ofstream(ofstream& out) const
{
    short   audio_format     = 1,
            block_align      = num_channels * (bits_per_sample / 8);

    int     i,
            sub_chunk_1_size = 28 + (num_channels * sizeof(int)),
            sub_chunk_2_size = num_samples * num_channels * (bits_per_sample / 8),
            byte_rate        = sample_rate * num_channels * (bits_per_sample / 8),
            chunk_size       = 20 + sub_chunk_1_size + sub_chunk_2_size;
    //------------------------------------------------------------------------
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "\nnum_samples      : " << num_samples;
#endif

    out.put('R');
    out.put('I');
    out.put('F');
    out.put('F');

    out.put((char) (chunk_size & 0x000000ff)       );
    out.put((char)((chunk_size & 0x0000ff00) >> 8 ));
    out.put((char)((chunk_size & 0x00ff0000) >> 16));
    out.put((char)((chunk_size & 0xff000000) >> 24));
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "\nchunk_size       : " << chunk_size;
#endif

    out.put('W');
    out.put('A');
    out.put('V');
    out.put('E');

    out.put('f');
    out.put('m');
    out.put('t');
    out.put(' ');

    out.put((char) (sub_chunk_1_size & 0x000000ff)       );
    out.put((char)((sub_chunk_1_size & 0x0000ff00) >> 8 ));
    out.put((char)((sub_chunk_1_size & 0x00ff0000) >> 16));
    out.put((char)((sub_chunk_1_size & 0xff000000) >> 24));
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "\nsub_chunk_1_size : " << sub_chunk_1_size;
#endif

    out.put((char) (audio_format & 0x00ff)      );
    out.put((char)((audio_format & 0xff00) >> 8)); // 2
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "\naudio_format     : " << audio_format;
#endif

    out.put((char) (num_channels & 0x00ff)      );
    out.put((char)((num_channels & 0xff00) >> 8)); // 4
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "\nnum_channels     : " << num_channels;
#endif

    out.put((char) (sample_rate & 0x000000ff)       );
    out.put((char)((sample_rate & 0x0000ff00) >> 8 ));
    out.put((char)((sample_rate & 0x00ff0000) >> 16));
    out.put((char)((sample_rate & 0xff000000) >> 24)); // 8
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "\nsample_rate      : " << sample_rate;
#endif

    out.put((char) (byte_rate & 0x000000ff)       );
    out.put((char)((byte_rate & 0x0000ff00) >> 8 ));
    out.put((char)((byte_rate & 0x00ff0000) >> 16));
    out.put((char)((byte_rate & 0xff000000) >> 24)); // 12
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "\nbyte_rate        : " << byte_rate;
#endif

    out.put((char) (block_align & 0x00ff)      );
    out.put((char)((block_align & 0xff00) >> 8)); // 14
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "\nblock_align      : " << block_align;
#endif

    out.put((char) (bits_per_sample & 0x00ff)      );
    out.put((char)((bits_per_sample & 0xff00) >> 8)); // 16
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "\nbits_per_sample  : " << bits_per_sample;
#endif

//----------------------------------------------------------------------------
// sample offsets added for LaserBoy !!!
//----------------------------------------------------------------------------
    out.put('o');
    out.put('f');
    out.put('f');
    out.put('s');
    out.put('e');
    out.put('t');
    out.put('s');
    out.put(' '); // 24
//----------------------------------------------------------------------------
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "\noffsets ..........";
#endif

    for(i = 0; i < num_channels; i++)
    {
        out.put((char)( offset[i] & 0x000000ff)       );
        out.put((char)((offset[i] & 0x0000ff00) >> 8 ));
        out.put((char)((offset[i] & 0x00ff0000) >> 16));
        out.put((char)((offset[i] & 0xff000000) >> 24)); // 24 + 4 * num_channels
#if EZ_ILDA_DEBUG_MESSAGES
        cout << "\noffset[" << i << "]        : " << offset[i];
#endif
    }
//----------------------------------------------------------------------------
    out.put((char) (ez_wave_mode & 0x000000ff)       );
    out.put((char)((ez_wave_mode & 0x0000ff00) >> 8 ));
    out.put((char)((ez_wave_mode & 0x00ff0000) >> 16));
    out.put((char)((ez_wave_mode & 0xff000000) >> 24));
//----------------------------------------------------------------------------
    out.put('d');
    out.put('a');
    out.put('t');
    out.put('a');
    
    out.put((char) (sub_chunk_2_size & 0x000000ff)       );
    out.put((char)((sub_chunk_2_size & 0x0000ff00) >> 8 ));
    out.put((char)((sub_chunk_2_size & 0x00ff0000) >> 16));
    out.put((char)((sub_chunk_2_size & 0xff000000) >> 24));
#if EZ_ILDA_DEBUG_MESSAGES
    cout << "\nsub_chunk_2_size : " << sub_chunk_2_size << endl << endl;
#endif
    return;
}

//############################################################################
//void ez_wave_header::tell() const
//{
//    int i;
//    cout << endl
//         << endl << "num_samples     : " << num_samples
//         << endl << "sample_rate     : " << sample_rate
//         << endl << "ez_wave_mode    : " << ((ez_wave_mode & EZ_WAVE_POLARITY    ) ? ("positive signal") : ("negative signal"))
//                                         << ((ez_wave_mode & EZ_WAVE_END_OF_FRAME) ? ("\n                  end of frame marks") : (""))
//                                         << ((ez_wave_mode & EZ_WAVE_UNIQUE_FRAME) ? ("\n                  unique frame marks") : (""))
//                                         << ((ez_wave_mode & EZ_WAVE_UNIQUE_POINT) ? ("\n                  unique point marks") : (""))
//         << endl << "num_channels    : " << num_channels
//         << endl << "bits_per_sample : " << bits_per_sample
//         << endl << "playing time    : " << ((double)num_samples / sample_rate) << " seconds";
//    for(i = 0; i < num_channels; i++)
//         cout << endl << "offset["<< i << "]       : " << offset[i];
//    cout << endl << endl;
//}

//############################################################################
//////////////////////////////////////////////////////////////////////////////
//############################################################################



