//###################################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//###################################################################################
#ifndef __EZ_WAVE_DEFINITIONS__
#define __EZ_WAVE_DEFINITIONS__

//###################################################################################
#include "ez_ilda_macros.hpp"

//###################################################################################
class ez_wave_header
{
public:
    ez_wave_header( const int    s = 0,
                    const int    r = 48000,
                    const short  n = 6,
                    const short  b = 16
                  );
    //-------------------------------------------------------------------------------
   ~ez_wave_header();
    //-------------------------------------------------------------------------------
    ez_wave_header(const ez_wave_header& header);
    //-------------------------------------------------------------------------------
    ez_wave_header& operator = (const ez_wave_header& header);
    //-------------------------------------------------------------------------------
    int      from_ifstream  (ifstream& in);
    void     to_ofstream    (ofstream& out) const ;
    //-------------------------------------------------------------------------------
    int      num_samples     ,
             sample_rate     ,
             ez_wave_mode    ,
            *offset          ;

    short    num_channels    ,
             bits_per_sample ;
};

//###################################################################################










//###################################################################################
// class ez_wave_sample
//###################################################################################
class ez_wave_sample
{
public:
    ez_wave_sample(int n = 6)
        {
            num_channels = n;
            channel = new short[num_channels];
            memset((void*)channel, 0x0000, num_channels * sizeof(short));
        }
    //-------------------------------------------------------------------------------
   ~ez_wave_sample()    {    delete channel;    }
    //-------------------------------------------------------------------------------
    int from_ifstream(ifstream& in)
        {
            if(!in.eof())
            {
                in.read((char*)channel, num_channels * sizeof(signed short));
                return 0;
            }
            return 1;
        }
    //-------------------------------------------------------------------------------
    int from_ifstream_inverted(ifstream& in)
        {
            if(!in.eof())
            {
                in.read((char*)channel, num_channels * sizeof(signed short));
                negate();
                return 0;
            }
            return 1;
        }
    //-------------------------------------------------------------------------------
    ez_wave_sample& operator = (const ez_wave_sample& sample)
                    {
                        delete channel;
                        num_channels = sample.num_channels;
                        channel = new short[num_channels];
                        memcpy((void*)channel, (void*)sample.channel, num_channels * sizeof(short));
                        return *this;
                    }
    //-------------------------------------------------------------------------------
    void negate()
            {
                for(int i = 0; i < num_channels; i++)
                    channel[i] = -channel[i];
                return;
            }
    //-------------------------------------------------------------------------------
    signed short   *channel     ;
    int             num_channels;
};

//###################################################################################
#endif

//###################################################################################
/////////////////////////////////////////////////////////////////////////////////////
//###################################################################################




